@@ -1,12 +1,19 @@
 from enum import IntEnum
+from functools import cache
 from pprint import pformat
 from re import match
+from socket import gethostname
 from typing import Dict, Any, Optional, Union, List
 
-from robotnikmq import Topic, Message, RobotnikConfig
+from robotnikmq import Topic, Message, RobotnikConfig, log
 from typeguard import typechecked
 
 
+@cache
+def hostname() -> str:
+    return gethostname().lower()
+
+
 @typechecked
 class Priority(IntEnum):
     INFO = 0
@@ -55,6 +62,12 @@ def priority_of(candidate: Union[str, int]) -> Priority:
 
 
 class NotificationMsg:
+    """This is the default message type from which all other types inherit. It
+       has the most basic and essential properties of a Legion message. Most of
+       the properties are Optional because they're not necessary for a message
+       type of "Notification". You probably should not be using this type
+       directly without setting the priorities and TTLs that you want.
+    """
     @typechecked
     def __init__(
         self,
@@ -64,7 +77,8 @@ class NotificationMsg:
         ttl: Optional[int] = None,
         priority: Optional[Priority] = None,
     ):
-        self.contents = contents
+        default_contents: Dict[str, Any] = {'msg_src': hostname()}
+        self.contents = {**default_contents, **contents}
         self.alert_key = (
             "[" + "][".join(alert_key) + "]"
             if isinstance(alert_key, List)
@@ -74,10 +88,38 @@ class NotificationMsg:
         self.ttl = ttl
         self.priority = priority or Priority.INFO
 
+    @property
+    def msg_src(self) -> Optional[str]:
+        """Returns the value of the msg_src property, which is typically the
+           hostname of the machine that generated the message, however, this
+           value can be overridden by settings it in the contents dict.
+
+        Returns:
+            Optional[str]: returns the string stored in contents['msg_src'] if
+                           its present in the contents dict or None if its not.
+        """
+        return self.contents.get('msg_src')
+
     @typechecked
     def broadcast(
         self, exchange: str, route: str, config: Optional[RobotnikConfig] = None
     ) -> None:
+        """Broadcasts the message to a given exchange and route, with an
+           optional Robotnik configuration which can be used to broadcast the
+           message to a wholly different set of servers than the one that would
+           normally be used by the RobotnikMQ library.
+
+        Args:
+            exchange (str): The topic exchange to broadcast to. See:
+                            https://www.rabbitmq.com/tutorials/amqp-concepts.html#exchanges
+            route (str): The routing key for the message. See:
+                         https://www.rabbitmq.com/tutorials/tutorial-four-python.html
+            config (Optional[RobotnikConfig], optional): An optional configuration that can be used
+                                                         to publish the message to a completely
+                                                         different set of RabbitMQ servers than what
+                                                         would otherwise be used by RobotnikMQ by
+                                                         default. Defaults to None.
+        """
         broadcast_msg(exchange, route, self, config)
 
 
@@ -127,6 +169,10 @@ class AlertComparison:
         return bool(match(*self.desc) or match(self.desc[1], self.desc[0]))
 
 
+def alert_key_str(key: Union[str, list[str]]) -> str:
+    return "[" + "][".join(key) + "]" if isinstance(key, List) else key
+
+
 class AlertMsg(NotificationMsg):
     @typechecked
     def __init__(
@@ -150,9 +196,7 @@ class AlertMsg(NotificationMsg):
             ttl=ttl or 30,
             contents=contents,
         )
-        self.alert_key: str = (
-            "[" + "][".join(key) + "]" if isinstance(key, List) else key
-        )
+        self.alert_key = alert_key_str(key)
         self.msg = from_msg
 
     @typechecked
@@ -341,7 +385,7 @@ def broadcast(
     _contents.update(contents)
     route += f".{priority.name.lower()}"
     Topic(exchange=exchange, config=config).broadcast(
-        Message(contents=_contents), routing_key=route
+        Message.of(contents=_contents), routing_key=route
     )
 
 
@@ -352,7 +396,8 @@ def broadcast_info(
     contents: Dict[str, Any],
     config: Optional[RobotnikConfig] = None,
 ):
-    broadcast(exchange, route, priority=Priority.INFO, contents=contents, config=config)
+    # broadcast(exchange, route, priority=Priority.INFO, contents=contents, config=config)
+    NotificationMsg(contents=contents).broadcast(exchange=exchange, route=route, config=config)
 
 
 @typechecked
